package cn.edu.whu.boot.xml.mapper;

import cn.edu.whu.boot.user.entities.User;
import cn.edu.whu.boot.xml.domain.Task;
import cn.edu.whu.boot.xml.domain.TaskExample;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TaskMapper {
    long countByExample(TaskExample example);

    int deleteByExample(TaskExample example);

    int deleteByPrimaryKey(Integer id);

    int insert(Task record);

    int insertSelective(Task record);

    List<Task> selectByExample(TaskExample example);

    Task selectByPrimaryKey(Integer id);

    int updateByExampleSelective(@Param("record") Task record, @Param("example") TaskExample example);

    int updateByExample(@Param("record") Task record, @Param("example") TaskExample example);

    int updateByPrimaryKeySelective(Task record);

    int updateByPrimaryKey(Task record);

    List<User> selectUsersByTaskId(int taskId);

    List<User> selectUsersByTableNameAndIdFieldName(@Param("tableName") String tableName, @Param("IdFieldName") String IdFieldName);
}